"""
Karma - handle karma (++ and --) tracking
Copyright (C) 2010  Brian S. Stephan

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import shelve

from Module import Module

__author__ = "Mike Bloy <mike@bloy.org>"
__date__ = "$Oct 23, 2010 11:12:33 AM$"

class Karma(Module):

    def __init__(self, config, server, modlist):
        """
        Upon creation, determine the save file location
        """

        Module.__init__(self, config, server, modlist)

        filename = self.config.get(self.__class__.__name__, 'karmafile')
        self.karmafile = filename + "_karma.dat"
        self.trendfile = filename + "_trends.dat"

        pattern = "(?:([a-zA-Z0-9_]+)|\(([a-zA-Z0-9_ ]+)\))"

        karmapattern = '^' + pattern + '(\+\+|--)'
        querypattern = '^!rank\s+(.*)'

        self.karmare = re.compile(karmapattern)
        self.queryre = re.compile(querypattern)

    def do(self, connection, event, nick, userhost, replypath, what, admin_unlocked):
        """look for karma strings at the start of messages"""

        if (self.karmare.match(what)):
            self.handle_karma_change(connection, nick, userhost, replypath, what)

        elif (self.queryre.match(what)):
            self.handle_karma_query(connection, nick, userhost, replypath, what)

    def handle_karma_change(self, connection, nick, userhost, replypath, what):
        """
        handle the karma change and storage.
        """
        match = self.karmare.match(what)
        key = match.group(1) if match.group(1) else match.group(2)
        value = match.group(3)
        if (value == '++'):
            value = 1;
        else:
            value = -1;

        # do karma recording
        karma = shelve.open(self.karmafile)
        try:
            oldvalue = 0;
            if karma.has_key(key):
                oldvalue = karma[key]
            newvalue = oldvalue + value;
            karma[key] = newvalue;
        finally:
            karma.close()

        trend = shelve.open(self.trendfile)
        try:
            nickpos = nick + "_pos"
            nickneg = nick + "_neg"
            trend_pos = 0;
            trend_neg = 0;
            if trend.has_key(nickpos):
                trend_pos = trend[nickpos]
            if trend.has_key(nickneg):
                trend_neg = trend[nickneg]
            if value > 0:
                trend_pos = trend_pos + 1
            else:
                trend_neg = trend_neg + 1
            trend[nickpos] = trend_pos;
            trend[nickneg] = trend_neg;
        finally:
            trend.close();

        reply = "karma change for '" + key + "' (" + str(value) + ") by " + nick
        self.reply(connection, replypath, reply)

    def handle_karma_query(self, connection, nick, userhost, replypath, what):
        match = self.queryre.match(what)
        key = match.group(1)

        karma = shelve.open(self.karmafile, "r")
        reply = key + ' has no karma'
        try:
            if karma.has_key(key):
                value = karma[key]
                reply = key + ' has ' + str(value) + ' points of karma'
        finally:
            karma.close()

        self.reply(connection, replypath, reply)

if __name__ == "__main__":
    print "Hello World"
